#  Copyright (C) 2025
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import types
import typing

TypingGenericAlias: typing.TypeAlias = type(typing.List[str])
AnyType: typing.TypeAlias = type | types.UnionType | types.GenericAlias | TypingGenericAlias  # OR typing.Annotated
